function stats = TauBatch(setname,n)
% TAUBATCH   Run problem instance for 20 values of tau

if nargin < 2, idxset = 1:20; end;


% Set solver names, default solvers and options
solvers = {'bgs','spg','spg-base','pg','smooth'};
solverLambda = 1; % Default solver for lambda formulation
solverTau    = 2; % Default solver for tau formulation
solverXnorm  = 2; % Solver to compare solution to

opts = struct();
opts.verbosity = 0;
opts.maxiter   = 1e4;
opts.maxfunc   = 5000;
opts.bgstol    = 1e-4;
opts.epsl1tol  = 1e-8;
opts.epsilon   = 5e-10;

[pathstr,name,ext,versn] = fileparts(mfilename('fullpath'));


% Create problem generator and instantiate first problem
generate  = QPproblem(setname);
[A,b,x0,groups,weights,lambda,tauBase] = generate(1);

% Check if tau is available
if isnan(tauBase)
   error('Problem formulation must be in tau');
end

% Initialize results
stats = cell(length(solvers),length(idxset));
scale = linspace(0,1,21); scale = scale(2:end);

for i=idxset

   % Create filename
   filename = sprintf('%s%sdata%sTau_%s_%03d.mat', ...
                      pathstr,filesep,filesep,upper(setname),i);

   % Check if solution already exists
   if exist(filename)
      fprintf('%s Instance %4d: Loading %s . . .\n',upper(setname),i,filename);
      data = load(filename,'info');
      info = data.info;
   else
      fprintf('%s Instance %4d: Solving . . .\n',upper(setname),i);

      % Initialize
      tau  = tauBase * scale(i);
      x    = cell(length(solvers),1); 
      info = cell(length(solvers),1);
   
      % Run first solver (must have tau formulation)
     [x{solverTau},info{solverTau}] = ...
          QPrun(A,b,groups,weights,lambda,tau,solvers{solverTau},opts);

     % Compute value of lambda and compute remaining solutions
     if isa(A,'function_handle')
        r = b - A(x{solverTau},1);
        Atr = A(r,2);
     else          
        r = b - A*x{solverTau};
        Atr = (r'*A)';
     end
     lambda = max(sqrt(accumarray(groups,Atr.^2))./weights);
     
     disp([lambda tau]);
      
     % Create list of remaining solvers
     jIdx = setdiff(1:length(solvers),solverTau);

     for j=jIdx
        [x{j},info{j}] = QPrun(A,b,groups,weights,lambda,tau, ...
                               solvers{j},opts);
     end
      
      
      % Compare the norm of the solutions
      for j=1:length(solvers)
         info{j}.normDiff = norm(x{j}-x{solverXnorm});
      end

      % Store results
      save(filename,'info');
      
        figure(1); clf; gca; hold on;
        marker = {'r.','b.','ro','bo','g.'};
        for k=1:length(solvers)
          plot(x{k},marker{k});
        end
        plot(x0,'kx'); pause(1);
   end
   
   stats(:,i) = info;
end
