function [w,v] = UGM_train(objType,options,X,Xedge,y,edgeStruct,infoStruct,varargin)
% Example of pseudolikelihood training:
%   [w,v]=UGM_train([],X,Xedge,y,edgeStruct,infoStruct)
% Example of loopy training:
%   options.objType = @UGM_Loss
%   [w,v]=UGM_train(options,X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP)
%
% Right now this function assumes all nodes have same number of states

[verbose,wInit,vInit,nonNegEdge,nodeRegType,nodeRegStr,edgeRegType,edgeRegStr] = ...
    myProcessOptions(options,'verbose',1,...
    'wInit',[],'vInit',[],'nonNegEdge',0,...
    'nodeRegType',2,'nodeRegStr',1e-4,'edgeRegType',2,'edgeRegStr',1e-4);

if ~isscalar(nodeRegStr) || ~isscalar(edgeRegStr)
    options.verbose = 0;
    [nodeRegStr,edgeRegStr] = CV2(objType,options,X,Xedge,y,edgeStruct,infoStruct,varargin{:})
    nodeRegStr = nodeRegStr*2;
    edgeRegStr = edgeRegStr*2;
end

if verbose
    options.Display = 'iter';
else
    options.Display = 'none';
end

if isempty(wInit) || isempty(vInit)
    [wInit,vInit] = UGM_initWeights(infoStruct,'zeros');
end

nNodeVars = numel(wInit);
nEdgeVars = numel(vInit);

lambdaNode = nodeRegStr*ones(size(wInit));
lambdaNode(1,:) = 0; % Don't penalize bias
lambdaEdge = edgeRegStr*ones(size(vInit));
lambdaEdge(1,:) = 0; % Don't penalize bias

if nodeRegType == 2 && edgeRegType == 2
    if nonNegEdge == 0
        % L2-Penalties, No Constraints
        wv = minFunc(@penalizedL2,[wInit(:);vInit(:)],options,objType,[lambdaNode(:);lambdaEdge(:)],X,Xedge,y,edgeStruct,infoStruct,varargin{:});
    else
        % L2-Penalties, Non-negative edge weights
        UB = inf(size([wInit(:);vInit(:)]));
        LB = [-inf(size(wInit(:)));zeros(size(vInit(:)))];
        wv = minFuncBC(@penalizedL2,[wInit(:);vInit(:)],LB,UB,options,objType,[lambdaNode(:);lambdaEdge(:)],X,Xedge,y,edgeStruct,infoStruct,varargin{:});
    end
else
    % L1-Penalty on Node and/or Edge weights

    % Split positive/negative parts of variables
    w = zeros(2*nNodeVars,1);
    w(1:nNodeVars) = wInit.*(wInit > 0);
    w(nNodeVars+1:2*nNodeVars) = -wInit.*(wInit < 0);
    v = zeros(2*nEdgeVars,1);
    v(1:nEdgeVars) = vInit.*(vInit > 0);
    v(nEdgeVars+1:2*nEdgeVars) = -vInit.*(vInit < 0);

    % Set Up regularizer(s)
    if nodeRegType == 2
        lambdaNodeL2 = repmat(lambdaNode(:),[2 1]);
        lambdaNodeL1 = zeros(size(lambdaNode));
    else
        lambdaNodeL2 = zeros(nNodeVars*2,1);
        lambdaNodeL1 = lambdaNode;
    end
    if edgeRegType == 2
        lambdaEdgeL2 = repmat(lambdaEdge(:),[2 1]);
        lambdaEdgeL1 = zeros(size(lambdaEdge));
    else
        lambdaEdgeL2 = zeros(nEdgeVars*2,1);
        lambdaEdgeL1 = lambdaEdge;
    end

    % Bound Constraints used for L1-penalty
    UB = inf(size([w(:);v(:)]));
    LB = zeros(size([w(:);v(:)]));

    if nonNegEdge == 1
        % Non-Negative Edge Weights are added with additional bound
        % constraints
        UB(nNodeVars*2+nEdgeVars+1:end) = 0;
    end

    % Solve
    wv = minFuncBC(@penalizedL2,[w(:);v(:)],LB,UB,options,@nonNegGrad,[lambdaNodeL2(:);lambdaEdgeL2(:)],[lambdaNodeL1(:);lambdaEdgeL1(:)],objType,X,Xedge,y,edgeStruct,infoStruct,varargin{:});

    % Form Solution
    wvP = wv(1:nNodeVars+nEdgeVars);
    wvM = wv(nNodeVars+nEdgeVars+1:end);
    wv = wvP-wvM;
end
[w,v] = UGM_splitWeights(wv,infoStruct);
end

%% Code for Cross-Validation using validation set likelihood

function [minNodeLambda,minEdgeLambda] = CV2(objType,options,X,Xedge,y,edgeStruct,infoStruct,varargin)
% Cross-validation to pick best options.nodeRegStr and options.edgeRegStr
% among set

nTrain = size(X,1);
minScore = inf;
minNodeLambda = nan;
minEdgeLambda = nan;

edgeLambdaValues = options.edgeRegStr;
nodeLambdaValues = options.nodeRegStr;

for edgeLambda = edgeLambdaValues
    for nodeLambda = nodeLambdaValues
        fprintf('NodeLambda = %.3f, EdgeLambda = %.3f',nodeLambda,edgeLambda);
        testScore = 0;
        for split = 1:2

            % Make Split
            if split == 1
                trainNdx = 1:floor(nTrain/2);
                testNdx = floor(nTrain/2)+1:nTrain;
            else
                trainNdx = floor(nTrain/2)+1:nTrain;
                testNdx = 1:floor(nTrain/2);
            end

            % Set Value
            options.nodeRegStr = nodeLambda;
            options.edgeRegStr = edgeLambda;

            % Train
            [w,v] = UGM_train(objType,options,X(trainNdx,:,:),Xedge(trainNdx,:,:),y(trainNdx,:),edgeStruct,infoStruct,varargin{:});

            % Test
            testScore = testScore + objType([w(:);v(:)],X(testNdx,:,:),Xedge(testNdx,:,:),y(testNdx,:),edgeStruct,infoStruct,varargin{:});

        end

        if testScore < minScore
            minScore = testScore;
            minNodeLambda = nodeLambda;
            minEdgeLambda = edgeLambda;
            fprintf(' (score = %.3f - new best lambda)\n',testScore);
        else
            fprintf(' (score = %.3f)\n',testScore);
        end
    end
end
end