function [x,iter] = bcdGaussian(A,b,lambda,groups,weights)
% BCDGAUSSIAN   Optimize least squares plus group one-norm penalty
%
%   GROUPS = group numbers per element of x, 1:ngroups;
%

groupIdx = unique(groups);
nGroups  = length(groupIdx);
[m,n]    = size(A);

x = ones(n,1);

objective = @(x) grpObjective(A,b,weights,lambda,groups,[],x,[]);


iter = 0; noprogress = 0; block = 1;
while (1)
  
  % Get block information
  k        = groupIdx(block);
  idx      = (groups == k);
     
  % Compute gradient with local solution set to zero
  xbar = x; xbar(idx) = 0;
  r    = A*xbar - b;
  g    = (r' * A(:,idx))';
  xold = x(idx);

  if norm(g) < lambda * weights(k)
     xlocal = zeros(sum(idx),1);
  else
     % Determine initial point
     if norm(x(idx)) > 1e-5
        xinit = x(idx);
     else
        xinit = ones(sum(idx),1);
     end
        
     % Optimize local problem
     localFun = @(xgrp) grpObjective(A,b,weights,lambda,groups,k,x,xgrp);
     options  = optimset('Display','off');
     options.GradObj = 'on';
     xlocal   = fminunc(localFun,xinit,options);
  end
  
  % Update the local solution
  x(idx) = xlocal;

  % Check change in x
  if (norm(xold - xlocal) > 1e-4)
     noprogress = 0;
  else
     noprogress = noprogress + 1;
     if noprogress == nGroups, break; end;
  end;

  % Display progress
  f = objective(x);
  fprintf('%3d %12.6e\n',iter,f);

  % Update iteration counter and block number
  iter  = iter + 1;
  block = mod(block,nGroups) + 1;
end





function [f,g] = grpObjective(A,b,w,lambda,groups,grp,x,xgrp)

if isempty(grp)
   r = A*x - b;
   f = sum(r.^2) / 2 + lambda * sum(w.*sqrt(accumarray(groups,x.^2)));
   g = 0;
else
   idx = groups == grp;
   x(idx) = xgrp;
   r = A*x - b;
   n = norm(xgrp,2);
   f = sum(r.^2) / 2 + lambda * w(grp) * n;
   g = (r'*A)' + lambda * w(grp) * x / n;
   g = g(idx);
end
